@@ -24,9 +24,8 @@
 
 #include "pango-renderer.h"
 #include "pango-impl-utils.h"
-#include "pango-layout-private.h"
 
-#define N_RENDER_PARTS 5
+#define N_RENDER_PARTS 4
 
 #define PANGO_IS_RENDERER_FAST(renderer) (renderer != NULL)
 #define IS_VALID_PART(part) ((guint)part < N_RENDER_PARTS)
@@ -46,10 +45,6 @@ struct _LineState
 
   gboolean strikethrough;
   PangoRectangle strikethrough_rect;
-  int strikethrough_glyphs;
-
-  PangoOverline  overline;
-  PangoRectangle overline_rect;
 
   int logical_rect_end;
 };
@@ -58,11 +53,9 @@ struct _PangoRendererPrivate
 {
   PangoColor color[N_RENDER_PARTS];
   gboolean color_set[N_RENDER_PARTS];
-  guint16 alpha[N_RENDER_PARTS];
 
   PangoLayoutLine *line;
   LineState *line_state;
-  PangoOverline overline;
 };
 
 static void pango_renderer_finalize                     (GObject          *gobject);
@@ -111,8 +104,7 @@ to_device (PangoMatrix *matrix,
     }
 }
 
-G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PangoRenderer, pango_renderer, G_TYPE_OBJECT,
-                                  G_ADD_PRIVATE (PangoRenderer))
+G_DEFINE_ABSTRACT_TYPE (PangoRenderer, pango_renderer, G_TYPE_OBJECT)
 
 static void
 pango_renderer_class_init (PangoRendererClass *klass)
@@ -126,12 +118,16 @@ pango_renderer_class_init (PangoRendererClass *klass)
   klass->prepare_run = pango_renderer_default_prepare_run;
 
   gobject_class->finalize = pango_renderer_finalize;
+
+  g_type_class_add_private (gobject_class, sizeof (PangoRendererPrivate));
 }
 
 static void
 pango_renderer_init (PangoRenderer *renderer)
 {
-  renderer->priv = pango_renderer_get_instance_private (renderer);
+  renderer->priv = G_TYPE_INSTANCE_GET_PRIVATE (renderer,
+						PANGO_TYPE_RENDERER,
+						PangoRendererPrivate);
   renderer->matrix = NULL;
 }
 
@@ -165,7 +161,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
 			    int               x,
 			    int               y)
 {
-  PangoLayoutIter iter;
+  PangoLayoutIter *iter;
 
   g_return_if_fail (PANGO_IS_RENDERER (renderer));
   g_return_if_fail (PANGO_IS_LAYOUT (layout));
@@ -182,7 +178,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
 
   pango_renderer_activate (renderer);
 
-  _pango_layout_get_iter (layout, &iter);
+  iter = pango_layout_get_iter (layout);
 
   do
     {
@@ -190,19 +186,19 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
       PangoLayoutLine *line;
       int              baseline;
 
-      line = pango_layout_iter_get_line_readonly (&iter);
+      line = pango_layout_iter_get_line_readonly (iter);
 
-      pango_layout_iter_get_line_extents (&iter, NULL, &logical_rect);
-      baseline = pango_layout_iter_get_baseline (&iter);
+      pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
+      baseline = pango_layout_iter_get_baseline (iter);
 
       pango_renderer_draw_layout_line (renderer,
 				       line,
 				       x + logical_rect.x,
 				       y + baseline);
     }
-  while (pango_layout_iter_next_line (&iter));
+  while (pango_layout_iter_next_line (iter));
 
-  _pango_layout_iter_destroy (&iter);
+  pango_layout_iter_free (iter);
 
   pango_renderer_deactivate (renderer);
 }
@@ -221,17 +217,15 @@ draw_underline (PangoRenderer *renderer,
     case PANGO_UNDERLINE_NONE:
       break;
     case PANGO_UNDERLINE_DOUBLE:
-    case PANGO_UNDERLINE_DOUBLE_LINE:
       pango_renderer_draw_rectangle (renderer,
 				     PANGO_RENDER_PART_UNDERLINE,
 				     rect->x,
 				     rect->y + 2 * rect->height,
 				     rect->width,
 				     rect->height);
-      G_GNUC_FALLTHROUGH;
+      /* Fall through */
     case PANGO_UNDERLINE_SINGLE:
     case PANGO_UNDERLINE_LOW:
-    case PANGO_UNDERLINE_SINGLE_LINE:
       pango_renderer_draw_rectangle (renderer,
 				     PANGO_RENDER_PART_UNDERLINE,
 				     rect->x,
@@ -240,7 +234,6 @@ draw_underline (PangoRenderer *renderer,
 				     rect->height);
       break;
     case PANGO_UNDERLINE_ERROR:
-    case PANGO_UNDERLINE_ERROR_LINE:
       pango_renderer_draw_error_underline (renderer,
 					   rect->x,
 					   rect->y,
@@ -250,47 +243,22 @@ draw_underline (PangoRenderer *renderer,
     }
 }
 
-static void
-draw_overline (PangoRenderer *renderer,
-               LineState     *state)
-{
-  PangoRectangle *rect = &state->overline_rect;
-  PangoOverline overline = state->overline;
-
-  state->overline = PANGO_OVERLINE_NONE;
-
-  switch (overline)
-    {
-    case PANGO_OVERLINE_NONE:
-      break;
-    case PANGO_OVERLINE_SINGLE:
-      pango_renderer_draw_rectangle (renderer,
-				     PANGO_RENDER_PART_OVERLINE,
-				     rect->x,
-				     rect->y,
-				     rect->width,
-				     rect->height);
-      break;
-    }
-}
-
 static void
 draw_strikethrough (PangoRenderer *renderer,
 		    LineState     *state)
 {
   PangoRectangle *rect = &state->strikethrough_rect;
   gboolean strikethrough = state->strikethrough;
-  int num_glyphs = state->strikethrough_glyphs;
 
   state->strikethrough = FALSE;
 
   if (strikethrough)
     pango_renderer_draw_rectangle (renderer,
-                                   PANGO_RENDER_PART_STRIKETHROUGH,
-                                   rect->x,
-                                   rect->y / num_glyphs,
-                                   rect->width,
-                                   rect->height / num_glyphs);
+				   PANGO_RENDER_PART_STRIKETHROUGH,
+				   rect->x,
+				   rect->y,
+				   rect->width,
+				   rect->height);
 }
 
 static void
@@ -313,18 +281,6 @@ handle_line_state_change (PangoRenderer  *renderer,
       rect->width = 0;
     }
 
-  if (part == PANGO_RENDER_PART_OVERLINE &&
-      state->overline != PANGO_OVERLINE_NONE)
-    {
-      PangoRectangle *rect = &state->overline_rect;
-
-      rect->width = state->logical_rect_end - rect->x;
-      draw_overline (renderer, state);
-      state->overline = renderer->priv->overline;
-      rect->x = state->logical_rect_end;
-      rect->width = 0;
-    }
-
   if (part == PANGO_RENDER_PART_STRIKETHROUGH &&
       state->strikethrough)
     {
@@ -371,18 +327,6 @@ add_underline (PangoRenderer    *renderer,
     case PANGO_UNDERLINE_LOW:
       new_rect.y += ink_rect->y + ink_rect->height + underline_thickness;
       break;
-    case PANGO_UNDERLINE_SINGLE_LINE:
-    case PANGO_UNDERLINE_DOUBLE_LINE:
-    case PANGO_UNDERLINE_ERROR_LINE:
-      new_rect.y -= underline_position;
-      if (state->underline == renderer->underline)
-        {
-          new_rect.y = MAX (current_rect->y, new_rect.y);
-          new_rect.height = MAX (current_rect->height, new_rect.height);
-          current_rect->y = new_rect.y;
-          current_rect->height = new_rect.height;
-        }
-      break;
     }
 
   if (renderer->underline == state->underline &&
@@ -400,57 +344,6 @@ add_underline (PangoRenderer    *renderer,
     }
 }
 
-static void
-add_overline (PangoRenderer    *renderer,
-              LineState        *state,
-              PangoFontMetrics *metrics,
-              int               base_x,
-              int               base_y,
-              PangoRectangle   *ink_rect,
-              PangoRectangle   *logical_rect)
-{
-  PangoRectangle *current_rect = &state->overline_rect;
-  PangoRectangle new_rect;
-  int underline_thickness = pango_font_metrics_get_underline_thickness (metrics);
-  int ascent = pango_font_metrics_get_ascent (metrics);
-
-  new_rect.x = base_x + logical_rect->x;
-  new_rect.width = logical_rect->width;
-  new_rect.height = underline_thickness;
-  new_rect.y = base_y;
-
-  switch (renderer->priv->overline)
-    {
-    case PANGO_OVERLINE_NONE:
-      g_assert_not_reached ();
-      break;
-    case PANGO_OVERLINE_SINGLE:
-      new_rect.y -= ascent;
-      if (state->overline == renderer->priv->overline)
-        {
-          new_rect.y = MIN (current_rect->y, new_rect.y);
-          new_rect.height = MAX (current_rect->height, new_rect.height);
-          current_rect->y = new_rect.y;
-          current_rect->height = new_rect.height;
-        }
-      break;
-    }
-
-  if (renderer->priv->overline == state->overline &&
-      new_rect.y == current_rect->y &&
-      new_rect.height == current_rect->height)
-    {
-      current_rect->width = new_rect.x + new_rect.width - current_rect->x;
-    }
-  else
-    {
-      draw_overline (renderer, state);
-
-      *current_rect = new_rect;
-      state->overline = renderer->priv->overline;
-    }
-}
-
 static void
 add_strikethrough (PangoRenderer    *renderer,
 		   LineState        *state,
@@ -458,8 +351,7 @@ add_strikethrough (PangoRenderer    *renderer,
 		   int               base_x,
 		   int               base_y,
 		   PangoRectangle   *ink_rect G_GNUC_UNUSED,
-		   PangoRectangle   *logical_rect,
-                   int               num_glyphs)
+		   PangoRectangle   *logical_rect)
 {
   PangoRectangle *current_rect = &state->strikethrough_rect;
   PangoRectangle new_rect;
@@ -469,22 +361,21 @@ add_strikethrough (PangoRenderer    *renderer,
 
   new_rect.x = base_x + logical_rect->x;
   new_rect.width = logical_rect->width;
-  new_rect.y = (base_y - strikethrough_position) * num_glyphs;
-  new_rect.height = strikethrough_thickness * num_glyphs;
+  new_rect.y = base_y - strikethrough_position;
+  new_rect.height = strikethrough_thickness;
 
-  if (state->strikethrough)
+  if (state->strikethrough &&
+      new_rect.y == current_rect->y &&
+      new_rect.height == current_rect->height)
     {
       current_rect->width = new_rect.x + new_rect.width - current_rect->x;
-      current_rect->y += new_rect.y;
-      current_rect->height += new_rect.height;
-      state->strikethrough_glyphs += num_glyphs;
     }
   else
     {
       draw_strikethrough (renderer, state);
+
       *current_rect = new_rect;
       state->strikethrough = TRUE;
-      state->strikethrough_glyphs = num_glyphs;
     }
 }
 
@@ -592,7 +483,6 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
   renderer->priv->line_state = &state;
 
   state.underline = PANGO_UNDERLINE_NONE;
-  state.overline = PANGO_OVERLINE_NONE;
   state.strikethrough = FALSE;
 
   text = G_LIKELY (line->layout) ? pango_layout_get_text (line->layout) : NULL;
@@ -627,7 +517,6 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
       else
 	{
 	  if (renderer->underline != PANGO_UNDERLINE_NONE ||
-              renderer->priv->overline != PANGO_OVERLINE_NONE ||
 	      renderer->strikethrough)
 	    {
 	      ink = &ink_rect;
@@ -685,7 +574,6 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	}
 
       if (renderer->underline != PANGO_UNDERLINE_NONE ||
-          renderer->priv->overline != PANGO_OVERLINE_NONE ||
 	  renderer->strikethrough)
 	{
 	  metrics = pango_font_get_metrics (run->item->analysis.font,
@@ -696,15 +584,10 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 			   x + x_off, y - rise,
 			   ink, logical);
 
-	  if (renderer->priv->overline != PANGO_OVERLINE_NONE)
-	    add_overline (renderer, &state,metrics,
-			   x + x_off, y - rise,
-			   ink, logical);
-
 	  if (renderer->strikethrough)
 	    add_strikethrough (renderer, &state, metrics,
 			       x + x_off, y - rise,
-			       ink, logical, run->glyphs->num_glyphs);
+			       ink, logical);
 
 	  pango_font_metrics_unref (metrics);
 	}
@@ -713,10 +596,6 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	  state.underline != PANGO_UNDERLINE_NONE)
 	draw_underline (renderer, &state);
 
-      if (renderer->priv->overline == PANGO_OVERLINE_NONE &&
-	  state.overline != PANGO_OVERLINE_NONE)
-	draw_overline (renderer, &state);
-
       if (!renderer->strikethrough && state.strikethrough)
 	draw_strikethrough (renderer, &state);
 
@@ -726,7 +605,6 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
   /* Finish off any remaining underlines
    */
   draw_underline (renderer, &state);
-  draw_overline (renderer, &state);
   draw_strikethrough (renderer, &state);
 
   renderer->priv->line_state = NULL;
@@ -822,7 +700,7 @@ pango_renderer_draw_glyph_item (PangoRenderer    *renderer,
 				int               x,
 				int               y)
 {
-  if (!text)
+  if (G_UNLIKELY (text))
     {
       pango_renderer_draw_glyphs (renderer,
 				  glyph_item->item->analysis.font,
@@ -1097,23 +975,17 @@ pango_renderer_default_draw_error_underline (PangoRenderer *renderer,
 					     int            width,
 					     int            height)
 {
-  int square;
-  int unit_width;
-  int width_units;
-  const PangoMatrix identity = PANGO_MATRIX_INIT;
+  int square = height / HEIGHT_SQUARES;
+  int unit_width = (HEIGHT_SQUARES - 1) * square;
+  int width_units = (width + unit_width / 2) / unit_width;
+  static const PangoMatrix identity = PANGO_MATRIX_INIT;
   const PangoMatrix *matrix;
   double dx, dx0, dy0;
   PangoMatrix total;
   int i;
 
-  if (width <= 0 || height <= 0)
-    return;
-
-  square = height / HEIGHT_SQUARES;
-  unit_width = (HEIGHT_SQUARES - 1) * square;
-  width_units = (width + unit_width / 2) / unit_width;
-
   x += (width - width_units * unit_width) / 2;
+  width = width_units * unit_width;
 
   if (renderer->matrix)
     matrix = renderer->matrix;
@@ -1272,7 +1144,6 @@ pango_renderer_deactivate (PangoRenderer *renderer)
  * @color: (allow-none): the new color or %NULL to unset the current color
  *
  * Sets the color for part of the rendering.
- * Also see pango_renderer_set_alpha().
  *
  * Since: 1.8
  **/
@@ -1311,9 +1182,9 @@ pango_renderer_set_color (PangoRenderer    *renderer,
  *
  * Gets the current rendering color for the specified part.
  *
- * Return value: (transfer none) (nullable): the color for the
- *   specified part, or %NULL if it hasn't been set and should be
- *   inherited from the environment.
+ * Return value: the color for the specified part, or %NULL
+ *  if it hasn't been set and should be inherited from the
+ *  environment.
  *
  * Since: 1.8
  **/
@@ -1330,59 +1201,6 @@ pango_renderer_get_color (PangoRenderer   *renderer,
     return NULL;
 }
 
-/**
- * pango_renderer_set_alpha:
- * @renderer: a #PangoRenderer
- * @part: the part to set the alpha for
- * @alpha: an alpha value between 1 and 65536, or 0 to unset the alpha
- *
- * Sets the alpha for part of the rendering.
- * Note that the alpha may only be used if a color is
- * specified for @part as well.
- *
- * Since: 1.38
- */
-void
-pango_renderer_set_alpha (PangoRenderer   *renderer,
-                          PangoRenderPart  part,
-                          guint16          alpha)
-{
-  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
-  g_return_if_fail (IS_VALID_PART (part));
-
-  if ((!alpha && !renderer->priv->alpha[part]) ||
-      (alpha && renderer->priv->alpha[part] &&
-       renderer->priv->alpha[part] == alpha))
-    return;
-
-  pango_renderer_part_changed (renderer, part);
-
-  renderer->priv->alpha[part] = alpha;
-}
-
-/**
- * pango_renderer_get_alpha:
- * @renderer: a #PangoRenderer
- * @part: the part to get the alpha for
- *
- * Gets the current alpha for the specified part.
- *
- * Return value: the alpha for the specified part,
- *   or 0 if it hasn't been set and should be
- *   inherited from the environment.
- *
- * Since: 1.38
- */
-guint16
-pango_renderer_get_alpha (PangoRenderer   *renderer,
-                          PangoRenderPart  part)
-{
-  g_return_val_if_fail (PANGO_IS_RENDERER_FAST (renderer), 0);
-  g_return_val_if_fail (IS_VALID_PART (part), 0);
-
-  return renderer->priv->alpha[part];
-}
-
 /**
  * pango_renderer_part_changed:
  * @renderer: a #PangoRenderer
@@ -1443,14 +1261,10 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
   PangoColor *fg_color = NULL;
   PangoColor *bg_color = NULL;
   PangoColor *underline_color = NULL;
-  PangoColor *overline_color = NULL;
   PangoColor *strikethrough_color = NULL;
-  guint16 fg_alpha = 0;
-  guint16 bg_alpha = 0;
   GSList *l;
 
   renderer->underline = PANGO_UNDERLINE_NONE;
-  renderer->priv->overline = PANGO_OVERLINE_NONE;
   renderer->strikethrough = FALSE;
 
   for (l = run->item->analysis.extra_attrs; l; l = l->next)
@@ -1463,10 +1277,6 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
 	  renderer->underline = ((PangoAttrInt *)attr)->value;
 	  break;
 
-	case PANGO_ATTR_OVERLINE:
-	  renderer->priv->overline = ((PangoAttrInt *)attr)->value;
-	  break;
-
 	case PANGO_ATTR_STRIKETHROUGH:
 	  renderer->strikethrough = ((PangoAttrInt *)attr)->value;
 	  break;
@@ -1483,22 +1293,10 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
 	  underline_color = &((PangoAttrColor *)attr)->color;
 	  break;
 
-	case PANGO_ATTR_OVERLINE_COLOR:
-	  overline_color = &((PangoAttrColor *)attr)->color;
-	  break;
-
 	case PANGO_ATTR_STRIKETHROUGH_COLOR:
 	  strikethrough_color = &((PangoAttrColor *)attr)->color;
 	  break;
 
-	case PANGO_ATTR_FOREGROUND_ALPHA:
-          fg_alpha = ((PangoAttrInt *)attr)->value;
-	  break;
-
-	case PANGO_ATTR_BACKGROUND_ALPHA:
-          bg_alpha = ((PangoAttrInt *)attr)->value;
-	  break;
-
 	default:
 	  break;
 	}
@@ -1507,9 +1305,6 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
   if (!underline_color)
     underline_color = fg_color;
 
-  if (!overline_color)
-    overline_color = fg_color;
-
   if (!strikethrough_color)
     strikethrough_color = fg_color;
 
@@ -1517,13 +1312,6 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
   pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, bg_color);
   pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, underline_color);
   pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, strikethrough_color);
-  pango_renderer_set_color (renderer, PANGO_RENDER_PART_OVERLINE, overline_color);
-
-  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_FOREGROUND, fg_alpha);
-  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_BACKGROUND, bg_alpha);
-  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_UNDERLINE, fg_alpha);
-  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_STRIKETHROUGH, fg_alpha);
-  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_OVERLINE, fg_alpha);
 }
 
 /**
@@ -1553,9 +1341,10 @@ pango_renderer_set_matrix (PangoRenderer     *renderer,
  * Gets the transformation matrix that will be applied when
  * rendering. See pango_renderer_set_matrix().
  *
- * Return value: (nullable): the matrix, or %NULL if no matrix has
- *  been set (which is the same as the identity matrix). The returned
- *  matrix is owned by Pango and must not be modified or freed.
+ * Return value: the matrix, or %NULL if no matrix has been set
+ *  (which is the same as the identity matrix). The returned
+ *  matrix is owned by Pango and must not be modified or
+ *  freed.
  *
  * Since: 1.8
  **/
@@ -1573,13 +1362,13 @@ pango_renderer_get_matrix (PangoRenderer *renderer)
  *
  * Gets the layout currently being rendered using @renderer.
  * Calling this function only makes sense from inside a subclass's
- * methods, like in its draw_shape vfunc, for example.
+ * methods, like in its draw_shape<!---->() for example.
  *
  * The returned layout should not be modified while still being
  * rendered.
  *
- * Return value: (transfer none) (nullable): the layout, or %NULL if
- *  no layout is being rendered using @renderer at this time.
+ * Return value: (transfer none): the layout, or %NULL if no layout is being
+ *  rendered using @renderer at this time.
  *
  * Since: 1.20
  **/
@@ -1598,13 +1387,13 @@ pango_renderer_get_layout (PangoRenderer *renderer)
  *
  * Gets the layout line currently being rendered using @renderer.
  * Calling this function only makes sense from inside a subclass's
- * methods, like in its draw_shape vfunc, for example.
+ * methods, like in its draw_shape<!---->() for example.
  *
  * The returned layout line should not be modified while still being
  * rendered.
  *
- * Return value: (transfer none) (nullable): the layout line, or %NULL
- *   if no layout line is being rendered using @renderer at this time.
+ * Return value: the layout line, or %NULL if no layout line is being
+ *  rendered using @renderer at this time.
  *
  * Since: 1.20
  **/
